@@ -4,9 +4,10 @@ Go to https://scratch.mit.edu/projects/578255313/ for the Scratch Encode/Decode
 """
 import json
 import requests
+from ssl import SSLEOFError, SSLError
 import websocket
 import time
-from pyemitter import Emitter
+from pymitter import EventEmitter
 from threading import Thread
 
 from scratchconnect import Exceptions
@@ -52,9 +53,10 @@ class CloudConnection:
             "accept": "application/json",
             "Content-Type": "application/json",
         }
-        self._make_connection()
+        self.event = EventEmitter()
         self.encoder = Encoder()
-        self.event = Emitter()
+        self._make_connection()
+        self._start_ping_thread()
 
     def get_variable_data(self, limit=100, offset=0):
         """
@@ -99,6 +101,24 @@ class CloudConnection:
         """
         self._ws.send(json.dumps(packet) + "\n")
 
+    def _ping_task(self):
+        """
+        Don't use this
+        """
+        while True:
+            self._ws.ping()
+            time.sleep(1)
+            self._ws.pong()
+            time.sleep(4)
+
+    def _start_ping_thread(self):
+        """
+        Don't use this
+        """
+        print("Pinging Thread Started!")
+        self.ping_task = Thread(target=self._ping_task, daemon=True)
+        self.ping_task.start()
+
     def _make_connection(self):
         """
         Don't use this
@@ -146,10 +166,8 @@ class CloudConnection:
             }
             self._send_packet(packet)
             return True
-        except ConnectionAbortedError or BrokenPipeError:
+        except (ConnectionAbortedError, BrokenPipeError, SSLEOFError, SSLError):
             self._make_connection()
-            time.sleep(0.1)
-            self.set_cloud_variable(variable_name, value)
             return False
 
     def encode(self, text):
@@ -200,4 +218,5 @@ class CloudConnection:
         """
         This feature was requested by @Ankit_Anmol on Scratch
         """
+        self._make_connection()
         Thread(target=self._event, args=(update_time,)).start()
