@@ -5,19 +5,22 @@ import requests
 import json
 import re
 
+from scratchconnect.UserCommon import UserCommon
 from scratchconnect import Exceptions
 from scratchconnect import Warnings
 from scratchconnect import Project
 from scratchconnect import Studio
 from scratchconnect import User
 from scratchconnect import Forum
+from scratchconnect.scScratchTerminal import _terminal
+from scratchconnect.scChart import _chart
 
 _website = "scratch.mit.edu"
 _login = f"https://{_website}/login/"
 _api = f"api.{_website}"
 
 
-class ScratchConnect:
+class ScratchConnect(UserCommon):
     def __init__(self, username=None, password=None, cookie=None, auto_cookie_login=False):
         """
         Class to make a connection to Scratch
@@ -31,10 +34,35 @@ class ScratchConnect:
 
         if self.username is not None and self.password is not None:
             self._login(cookie=False, auto_cookie_login=self.auto_cookie_login)
+            self._logged_in = True
         elif self.cookie is not None:
             self._login(cookie=True, auto_cookie_login=self.auto_cookie_login)
+            self._logged_in = True
         else:
-            raise Exceptions.InvalidInfo("Username or Password not given!")
+            self.headers = {
+                "x-csrftoken": "a",
+                "x-requested-with": "XMLHttpRequest",
+                "Cookie": "scratchcsrftoken=a;scratchlanguage=en;",
+                "referer": "https://scratch.mit.edu",
+                "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.101 Safari/537.36"
+            }
+            self._logged_in = False
+            self.session_id = ""
+            self.headers = {
+                "x-csrftoken": "",
+                "X-Token": "",
+                "x-requested-with": "XMLHttpRequest",
+                "Cookie": "scratchcsrftoken="
+                          + ""
+                          + ";scratchlanguage=en;scratchsessionsid="
+                          + ""
+                          + ";",
+                "referer": "https://scratch.mit.edu",
+            }
+            Warnings.LoginWarning(
+                'Login with Username/Password and Cookie Failed! Continuing without login...')
+        super().__init__(self.username,
+                         self.headers)  # Get other properties and methods from the parent(UserCommon) class
         self.update_data()
 
     def _login(self, cookie=False, auto_cookie_login=False):
@@ -55,7 +83,7 @@ class ScratchConnect:
             try:
                 request.json()
             except Exception:
-                if request.status_code == 403: # *
+                if request.status_code == 403:  # *
                     if auto_cookie_login is True:
                         self._cookie_login()
                     else:
@@ -113,80 +141,6 @@ class ScratchConnect:
         except KeyError:
             raise Exceptions.InvalidUser(f"Username '{username}' doesn't exist!")
 
-    def update_data(self):
-        """
-        Update the stored data
-        """
-        self.user_id = None
-        self.user_thumbnail_url = None
-        self.user_messages_count = None
-        self.user_messages = None
-        self.user_work = None
-        self.user_status = None
-        self.user_joined_date = None
-        self.user_country = None
-        self.user_featured_data = None
-        self.user_projects = None
-        self.user_followers_count = None
-        self.user_following_count = None
-        self.user_total_views = None
-        self.user_total_loves = None
-        self.user_total_faves = None
-        self.user_following = None
-        self.user_followers = None
-        self.user_favourites = None
-        self.user_projects_count = None
-
-        data = requests.get(f"https://{_api}/users/{self.username}").json()
-        try:
-            self.user_id = data["id"]
-        except KeyError:
-            raise Exceptions.InvalidUser(f"Username '{self.username}' doesn't exist!")
-        self.user_work = data["profile"]["status"]
-        self.user_bio = data["profile"]["bio"]
-        self.user_joined_date = data["history"]["joined"]
-        self.user_country = data["profile"]["country"]
-        self.user_thumbnail_url = data["profile"]["images"]
-
-    def _update_db_data(self):
-        """
-        Update the stored Data (DON'T USE)
-        """
-        data = requests.get(f"https://scratchdb.lefty.one/v3/user/info/{self.username}").json()
-        self.user_status = data["status"]
-        self.user_followers_count = data["statistics"]["followers"]
-        self.user_following_count = data["statistics"]["following"]
-        self.user_total_views = data["statistics"]["views"]
-        self.user_total_loves = data["statistics"]["loves"]
-        self.user_total_faves = data["statistics"]["favorites"]
-
-    def id(self):
-        """
-        Get the ID of a user's profile
-        """
-        if self.id is None:
-            self.update_data()
-        return self.user_id
-
-    def thumbnail_url(self):
-        """Return the thumbnail URL of a user"""
-        if self.user_thumbnail_url is None:
-            self.update_data()
-        return self.user_thumbnail_url
-
-    def messages_count(self):
-        """
-        Get the messages count of the logged in user
-        """
-        headers = {
-            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.101 Safari/537.36"}
-        if self.user_messages_count is None:
-            self.user_messages_count = \
-                requests.get(f"https://api.scratch.mit.edu/users/{self.username}/messages/count",
-                             headers=headers).json()[
-                    "count"]
-        return self.user_messages_count
-
     def messages(self, all=False, limit=20, offset=0, filter="all"):
         """
         Get the list of messages
@@ -196,6 +150,8 @@ class ScratchConnect:
         :param filter: Filter the messages
         :return: The list of the messages
         """
+        if self._logged_in is False:
+            raise Exceptions.UnauthorizedAction("Cannot perform the action because the user is not logged in!")
         headers = {
             "x-csrftoken": self.csrf_token,
             "X-Token": self.token,
@@ -228,203 +184,26 @@ class ScratchConnect:
             self.user_messages = messages
         return self.user_messages
 
-    def work(self):
+    def clear_messages(self):
         """
-        Returns the 'What I am working on' of a Scratch profile
+        Clear the messages
         """
-        if self.user_work is None:
-            self.update_data()
-        return self.user_work
+        if self._logged_in is False:
+            raise Exceptions.UnauthorizedAction("Cannot perform the action because the user is not logged in!")
+        return requests.post(f"https://scratch.mit.edu/site-api/messages/messages-clear/", headers=self.headers).text
 
-    def bio(self):
+    def my_stuff_projects(self, order="all", page=1, sort_by=""):
         """
-        Returns the 'About me' of a Scratch profile
+        Get the projects in the MyStuff section of the logged in user
+        :param order: the order
+        :param page: the page
+        :param sort_by: sort
         """
-        if self.user_bio is None:
-            self.update_data()
-        return self.user_bio
-
-    def status(self):
-        """
-        Returns the status(Scratcher or New Scratcher) of a Scratch profile
-        """
-        if self.user_status is None:
-            self._update_db_data()
-        return self.user_status
-
-    def joined_date(self):
-        """
-        Returns the joined date of a Scratch profile
-        """
-        if self.user_joined_date is None:
-            self.update_data()
-        return self.user_joined_date
-
-    def country(self):
-        """
-        Returns the country of a Scratch profile
-        """
-        if self.user_country is None:
-            self.update_data()
-        return self.user_country
-
-    def featured_data(self):
-        """
-        Returns the featured project data of the Scratch profile
-        """
-        if self.user_featured_data is None:
-            self.user_featured_data = requests.get(f"https://scratch.mit.edu/site-api/users/all/{self.username}").json()
-        return self.user_featured_data
-
-    def projects(self, all=False, limit=20, offset=0):
-        """
-        Returns the list of shared projects of a user
-        :param all: If you want all then set it to True
-        :param limit: The limit of the projects
-        :param offset: The number of projects to be skipped from the beginning
-        """
-        if self.user_projects is None:
-            projects = []
-            if all:
-                offset = 0
-                while True:
-                    request = requests.get(
-                        f"https://api.scratch.mit.edu/users/{self.username}/projects/?limit=40&offset={offset}").json()
-                    projects.append(request)
-                    if len(request) != 40:
-                        break
-                    offset += 40
-            if not all:
-                for i in range(1, limit + 1):
-                    request = requests.get(
-                        f"https://api.scratch.mit.edu/users/{self.username}/projects/?limit={limit}&offset={offset}").json()
-                    projects.append(request)
-            self.user_projects = projects
-        return self.user_projects
-
-    def projects_count(self):
-        if self.user_projects_count is None:
-            all_projects = self.projects(all=True)
-            count = 0
-            for i in all_projects:
-                count += len(i)
-            self.user_projects_count = count
-        return self.user_projects_count
-
-    def followers_count(self):
-        """
-        Returns the follower count of a user
-        """
-        if self.user_followers_count is None:
-            self._update_db_data()
-        return self.user_followers_count
-
-    def following_count(self):
-        """
-        Returns the following count of a user
-        """
-        if self.user_following_count is None:
-            self._update_db_data()
-        return self.user_following_count
-
-    def total_views(self):
-        """
-        Returns the total views count of all the shared projects of a user
-        """
-        if self.user_total_views is None:
-            self._update_db_data()
-        return self.user_total_views
-
-    def total_loves_count(self):
-        """
-        Returns the total loves count of all the shared projects of a user
-        """
-        if self.user_total_loves is None:
-            self._update_db_data()
-        return self.user_total_loves
-
-    def total_favourites_count(self):
-        """
-        Returns the total favourites count of all the shared projects of a user
-        """
-        if self.user_total_faves is None:
-            self._update_db_data()
-        return self.user_total_faves
-
-    def following(self, all=False, limit=20, offset=0):
-        """
-        Returns the list of the user following
-        :param all: If you want all then set it to True
-        :param limit: The limit of the users
-        :param offset: The number of users to be skipped from the beginning
-        """
-        if self.user_following is None:
-            following = []
-            if all:
-                offset = 0
-                while True:
-                    response = requests.get(
-                        f"https://api.scratch.mit.edu/users/{self.username}/following/?limit=40&offset={offset}").json()
-                    offset += 40
-                    following.append(response)
-                    if len(response) != 40:
-                        break
-            if not all:
-                response = requests.get(
-                    f"https://api.scratch.mit.edu/users/{self.username}/following/?limit={limit}&offset={offset}").json()
-                following.append(response)
-            self.user_following = following
-        return self.user_following
-
-    def followers(self, all=False, limit=20, offset=0):
-        """
-        Returns the list of the user followers
-        :param all: If you want all then set it to True
-        :param limit: The limit of the users
-        :param offset: The number of users to be skipped from the beginning
-        """
-        if self.user_followers is None:
-            followers = []
-            if all:
-                offset = 0
-                while True:
-                    response = requests.get(
-                        f"https://api.scratch.mit.edu/users/{self.username}/followers/?limit=40&offset={offset}").json()
-                    offset += 40
-                    followers.append(response)
-                    if len(response) != 40:
-                        break
-            if not all:
-                response = requests.get(
-                    f"https://api.scratch.mit.edu/users/{self.username}/followers/?limit={limit}&offset={offset}").json()
-                followers.append(response)
-            self.user_followers = followers
-        return self.user_followers
-
-    def favourites(self, all=False, limit=20, offset=0):
-        """
-        Returns the list of the user favourites
-        :param all: If you want all then set it to True
-        :param limit: The limit of the projects
-        :param offset: The number of projects to be skipped from the beginning
-        """
-        if self.user_favourites is None:
-            favourites = []
-            if all:
-                offset = 0
-                while True:
-                    response = requests.get(
-                        f"https://api.scratch.mit.edu/users/{self.username}/favorites/?limit=40&offset={offset}").json()
-                    offset += 40
-                    favourites.append(response)
-                    if len(response) != 40:
-                        break
-            if not all:
-                response = requests.get(
-                    f"https://api.scratch.mit.edu/users/{self.username}/favorites/?limit={limit}&offset={offset}").json()
-                favourites.append(response)
-            self.user_favourites = favourites
-        return self.user_favourites
+        if self._logged_in is False:
+            raise Exceptions.UnauthorizedAction("Cannot perform the action because the user is not logged in!")
+        return requests.get(
+            f"https://scratch.mit.edu/site-api/projects/{order}/?page={page}&ascsort=&descsort={sort_by}",
+            headers=self.headers).json()
 
     def toggle_commenting(self):
         """
@@ -493,28 +272,6 @@ class ScratchConnect:
                             headers=self.headers,
                             )
 
-    def all_data(self):
-        """
-        Returns all the data of the user
-        """
-        data = {
-            'UserName': self.username,
-            'UserId': self.id(),
-            'Messages Count': self.messages_count(),
-            'Join Date': self.joined_date(),
-            'Status': self.status(),
-            'Work': self.work(),
-            'Bio': self.bio(),
-            'Country': self.country(),
-            'Follower Count': self.followers_count(),
-            'Following Count': self.following_count(),
-            'Total Views': self.total_views(),
-            'Total Loves': self.total_loves_count(),
-            'Total Favourites': self.total_favourites_count(),
-            'Total Projects Count': self.projects_count()
-        }
-        return data
-
     def _check_project(self, project_id):
         """
         Don't use this function
@@ -524,6 +281,18 @@ class ScratchConnect:
         except KeyError:
             raise Exceptions.InvalidProject(f"The project with ID - '{project_id}' doesn't exist!")
 
+    def feed(self, limit=40, offset=0):
+        """
+        Returns the "What's Happening" section of the front page
+        :param limit: the limit; max: 40
+        :param offset: the offset
+        """
+        if self._logged_in is False:
+            raise Exceptions.UnauthorizedAction("Cannot perform the action because the user is not logged in!")
+        return requests.get(
+            f"https://api.scratch.mit.edu/users/{self.username}/following/users/activity?limit={limit}&offset={offset}",
+            headers=self.headers).json()
+
     def site_health(self):
         """
         Returns the health of the Scratch Website.
@@ -594,15 +363,6 @@ class ScratchConnect:
             + search
         ).text)
 
-    def comments(self, limit=5, page=1):
-        """
-        Get comments of the profile of the user
-        :param limit: The limit
-        :param page: The page
-        """
-        return requests.get(
-            f"https://scratch-comments-api.sid72020123.repl.co/user/?username={self.username}&limit={limit}&page={page}").json()
-
     def set_featured_project(self, project_id, label='featured_project'):
         """
         Set the 'Featured Project' of a Scratch Profile
@@ -615,6 +375,8 @@ class ScratchConnect:
                 "my_favorite_things": 3,
                 "why_i_scratch": 4,
         """
+        if self._logged_in is False:
+            raise Exceptions.UnauthorizedAction("Cannot perform the action because the user is not logged in!")
         self._check_project(project_id)
         if not requests.get(f"https://api.scratch.mit.edu/projects/{project_id}/").json()["author"][
                    "username"] == self.username:
@@ -636,21 +398,6 @@ class ScratchConnect:
                             headers=self.headers,
                             ).json()
 
-    def user_follower_history(self, segment="", range=30):
-        """
-        Return the follower history of the user
-        :param segment: The length of time between each segment, defaults to 1 day.
-        :param range: Of how far back to get history, defaults to 30 days
-        """
-        return requests.get(
-            f"https://scratchdb.lefty.one/v3/user/graph/{self.username}/followers?segment={segment}&range={range}").json()
-
-    def ocular_data(self):
-        """
-        Get ocular data of the user
-        """
-        return requests.get(f"https://my-ocular.jeffalo.net/api/user/{self.username}").json()
-
     def search_forum(self, q, order="relevance", page=0):
         """
         Search the forum
@@ -665,29 +412,41 @@ class ScratchConnect:
         Connect a Scratch User
         :param username: A valid Username
         """
-        return User.User(username=username, client_username=self.username, csrf_token=self.csrf_token,
-                         session_id=self.session_id, token=self.token)
+        return User.User(username=username, client_username=self.username, headers=self.headers,
+                         logged_in=self._logged_in)
 
     def connect_studio(self, studio_id):
         """
         Connect a Scratch Studio
         :param studio_id: A valid studio ID
         """
-        return Studio.Studio(id=studio_id, client_username=self.username, csrf_token=self.csrf_token,
-                             session_id=self.session_id, token=self.token)
+        return Studio.Studio(id=studio_id, client_username=self.username, headers=self.headers,
+                             logged_in=self._logged_in)
 
     def connect_project(self, project_id, access_unshared=False):
         """
         Connect a Scratch Project
         :param project_id: A valid project ID
+        :param access_unshared: Set to True if you want to connect an unshared project
         """
-        return Project.Project(id=project_id, client_username=self.username, csrf_token=self.csrf_token,
-                               session_id=self.session_id, token=self.token, unshared=access_unshared)
+        return Project.Project(id=project_id, client_username=self.username, headers=self.headers,
+                               logged_in=self._logged_in, unshared=access_unshared, session_id=self.session_id)
 
     def connect_forum_topic(self, forum_id):
         """
         Connect a Scratch Forum Topic
         :param forum_id: A valid forum topic ID
         """
-        return Forum.Forum(id=forum_id, client_username=self.username, csrf_token=self.csrf_token,
-                           session_id=self.session_id, token=self.token)
+        return Forum.Forum(id=forum_id, client_username=self.username, headers=self.headers, logged_in=self._logged_in)
+
+    def create_new_terminal(self):
+        """
+        Create a new Terminal object
+        """
+        return _terminal(sc=self)
+
+    def create_new_chart(self):
+        """
+        Create a new Chart object
+        """
+        return _chart(sc=self)
